import requests
from .PyTriliumClient import PyTriliumClient


class PyTriliumNoteClient(PyTriliumClient):
    def __init__(self, url, token, debug=False) -> None:
        super().__init__(url, token, debug)

    def get_note_by_id(self, note_id: str) -> dict:
        """Given the Note's ID, this will return the Note's information.

        Parameters
        ----------
        note_id : str
            Trilium's ID for the Note, this can be seen by clicking the 'i' on the note, near the top.

        Returns
        -------
        requests.Response
            The response from the Trilium API.
        """
        return self.make_request(f"/notes/{note_id}").json()

    def get_note_content_by_id(self, note_id: str) -> str:
        """Given the Note's ID, this will return the Note's content.

        Parameters
        ----------
        note_id : str
            Trilium's ID for the Note, this can be seen by clicking the 'i' on the note, near the top.

        Returns
        -------
        str
            The content of the note, most likely in HTML format.
        """
        return self.make_request(f"/notes/{note_id}/content").text
    
    def put_note_content_by_id(self, note_id: str, data: str) -> requests.Response:
        """Given the Note's ID, this will update the Note's content.

        Parameters
        ----------
        note_id : str
            Trilium's ID for the Note, this can be seen by clicking the 'i' on the note, near the top.
        data : str
            The data to send to the Trilium API. This should be in the format of a JSON string.

        Returns
        -------
        requests.Response
            The response from the Trilium API.
        """
        return self.make_request(f"/notes/{note_id}/content", method="PUT", data=data)

    def patch_note_by_id(self, note_id: str, data: str) -> requests.Response:
        """Given the Note's ID, this will update the Note's content.

        Parameters
        ----------
        note_id : str
            Trilium's ID for the Note, this can be seen by clicking the 'i' on the note, near the top.
        data : str
            The data to send to the Trilium API. This should be in the format of a JSON string.

        Returns
        -------
        requests.Response
            The response from the Trilium API.
        """
        return self.make_request(f"/notes/{note_id}", method="PATCH", data=data)

    def delete_note_by_id(self, note_id: str) -> requests.Response:
        """Given the Note's ID, this will delete the Note.

        Parameters
        ----------
        note_id : str
            Trilium's ID for the Note, this can be seen by clicking the 'i' on the note, near the top.

        Returns
        -------
        requests.Response
            The response from the Trilium API.
        """
        return self.make_request(f"/notes/{note_id}", method="DELETE")

    def export_note_by_id(self, note_id: str, filepath_to_save_export_zip: str, format="html") -> bool:
        """Given the Note's ID, export itself and all child notes into a singular .zip archive.

        Parameters
        ----------
        note_id : str
            Trilium's ID for the Note, this can be seen by clicking the 'i' on the note, near the top.
        filepath_to_save_export_zip : str
            The path of where to save the .zip archive that is generated by Trilium.
        format : str, optional
            The format to export the Notes in, by default "html". Can also be "markdown".

        Returns
        -------
        bool
            Returns True if the Note was exported successfully, and written to the path. Otherwise, returns False.
        """

        # If the filepath ends with a slash, this means that a directory was specified.
        # We should add the filename to the end of it.
        if filepath_to_save_export_zip.endswith("/"):
            filepath_to_save_export_zip += f"pytrilium_export_{note_id}.zip"

        # Make sure the filepath ends with .zip. If not, add it.
        if not filepath_to_save_export_zip.endswith(".zip"):
            filepath_to_save_export_zip += ".zip"

        try:
            params = {"format": format}
            response = self.make_request(f"/notes/{note_id}/export", params=params)

            with open(filepath_to_save_export_zip, "wb") as f:
                f.write(response.content)
        except Exception as e:
            print(e)
            return False
        return True

    def create_note_revision(self, note_id: str, data: str, format: str = "html") -> requests.Response:
        """Given the Note's ID, create a new revision of the Note.

        Parameters
        ----------
        note_id : str
            Trilium's ID for the Note, this can be seen by clicking the 'i' on the note, near the top.
        data : str
            The data to send to the Trilium API. This should be in the format of a JSON string.
        format : str, optional
            The format of which the data being provided is in. By default "html". Can also be "markdown".

        Returns
        -------
        requests.Response
            The response from the Trilium API.
        """

        params = {"format": format}
        return self.make_request(f"/notes/{note_id}/note-revision", method="POST", data=data, params=params)



from .PyTriliumNoteClient import PyTriliumNoteClient
from .PyTriliumBranchClient import PyTriliumBranchClient
from .PyTriliumAttributeClient import PyTriliumAttributeClient


class PyTrilium(PyTriliumNoteClient, PyTriliumBranchClient, PyTriliumAttributeClient):
    def __init__(self, url, token, debug=False) -> None:
        """Initializes the PyTrilium class.

        Parameters
        ----------
        url : str
            The URL of the Trilium instance. This should include the protocol (http:// or https://) and the port if it is not the protocol's respective port (443 for https, 80 for http). e.g. `https://trilium.example.com:8080`
        token : str
            The token for the Trilium instance. This can be found in the Trilium settings.
        debug : bool, optional
            If you would like to enable debugging, set this to True, by default False
        """
        super().__init__(url, token, debug)

        # Set up the requests session, and make a generic call to Trilium to validate that it works
        self.make_requests_session()
        self.attempt_basic_call()



import requests
from requests.adapters import HTTPAdapter, Retry

# Local import
from . import log


class PyTriliumClient:
    def __init__(self, url: str, token: str, debug: bool = False) -> None:
        """Initializes the PyTriliumClient class.

        Parameters
        ----------
        url : str
            The URL of the Trilium instance. This should include the protocol (http:// or https://) and the port if it is not the protocol's respective port (443 for https, 80 for http). e.g. `https://trilium.example.com:8080`
        token : str
            The token for the Trilium instance. This can be found in the Trilium settings.
        debug : bool, optional
            If you would like to enable debugging, set this to True, by default False

        Raises
        ------
        ValueError
            If the URL is invalid, this will raise a ValueError.
        """

        self.token = token
        if not self.clean_url(url):
            raise ValueError(
                "Invalid URL, please make sure to include https:// or http:// and that the URL is correct. The attempted URL was: "
                + self.url
            )

        self.logger = log.get_logger(
            logger_name="PyTriliumClient",
            log_file_name="PyTriliumClient.log",
            debug=debug,
            create_log_file=True,
        )

        # The valid response codes that can come from Triliu
        # everything else will be logged as a console warning
        self.valid_response_codes = [200, 201, 202, 204]

    def make_requests_session(self) -> None:
        """Creates a requests session with the token and user agent header."""
        self.session = requests.Session()
        self.session.headers.update({"User-Agent": "PyTriliumClient/0.0.1"})
        self.session.headers.update({"Authorization": self.token})

        # Set up retry logic
        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])

        # Have it work for both http and https
        self.session.mount("https://", HTTPAdapter(max_retries=retries))
        self.session.mount("http://", HTTPAdapter(max_retries=retries))

    def make_request(self, api_endpoint: str, method="GET", data="", params={}) -> requests.Response:
        """Standard request method for making requests to the Trilium API.

        Parameters
        ----------
        api_endpoint : str
            The API endpoint to make the request to. This should not include the URL or the /etapi prefix.
        method : str, optional
            The HTTP method to use, by default "GET"
        data : str, optional
            The body data to send with the request, by default ""
        params : dict, optional
            The parameters to include in the API call, by default {}

        Returns
        -------
        requests.Response
            The response from the Trilium API.
        """
        # We use our own session that holds the token, so we shouldn't
        # need to enforce it here.

        request_url = self.url + api_endpoint
        req_resp = self.session.request(method, request_url, data=data, params=params)
        if req_resp.status_code not in self.valid_response_codes:
            self.logger.warning(
                f"Possible invalid response code: {str(req_resp.status_code)}, response text: {req_resp.text}"
            )
        return req_resp

    def clean_url(self, url: str) -> bool:
        """Cleans the URL to make sure it is valid.


        Parameters
        ----------
        url : str
            The URL to clean.

        Returns
        -------
        bool
            If the URL is valid, this will return True. If the URL is invalid, this will return False.
        """
        if "/etapi" not in url:
            url = url + "/etapi"
        if "http" not in url and "https" not in url:
            # Then this URL is just scuffed
            return False
        self.url = url

        return True

    def attempt_basic_call(self) -> None:
        """Attempts a basic call to the Trilium API to make sure that the URL and token are valid."""
        resp = self.make_request("/app-info")
        if resp.status_code not in self.valid_response_codes:
            raise ValueError(
                f"Invalid response code: {str(resp.status_code)}, response text: {resp.text}. Response code should be one of {self.valid_response_codes}. Please check your Trilium, URL, and token."
            )



import logging
import sys
import coloredlogs
from logging.handlers import TimedRotatingFileHandler

FMT = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
DATE_FMT = "%m-%d-%Y %H:%M"

FILE_FORMATTER = logging.Formatter("%(asctime)s - %(funcName)s:%(lineno)d - %(name)s - %(levelname)s - %(message)s")
CONSOLE_FORMATTER = logging.Formatter(FMT)

# humanfriendly --demo
CUSTOM_FIELD_STYLES = {
    "asctime": {"color": "green"},
    "hostname": {"color": "magenta"},
    "levelname": {"bold": True, "color": "black"},
    "name": {"color": 200},
    "programname": {"color": "cyan"},
    "username": {"color": "yellow"},
}


def get_console_handler(debug):
    """
    Since we don't want to overwhelm or freak out the user, we're just going to send the output
    of debugging over to the file, and only send INFO out to the user.
    """

    # First, let's set the console StreamHandler
    console_handler = logging.StreamHandler(sys.stdout)

    # If debug is true, print it out to the screen.
    if debug == True:
        console_handler.setLevel(logging.DEBUG)
    else:
        console_handler.setLevel(logging.INFO)
    console_handler.setFormatter(CONSOLE_FORMATTER)
    return console_handler


def get_file_handler(debug=False, log_file_name="log.txt"):
    """
    We're going to print out the debug output to the log file.
    """

    file_handler = TimedRotatingFileHandler(log_file_name, when="midnight")

    # We want to print out debug information to this file.
    if debug:
        file_handler.setLevel(logging.DEBUG)
    else:
        file_handler.setLevel(logging.INFO)
    file_handler.setFormatter(FILE_FORMATTER)
    return file_handler


def get_logger(
    logger_name="Template Repository Logger",
    log_file_name="log.txt",
    debug=False,
    create_log_file=True,
):
    """Get the logger, for the current namespace.

    Args:
        logger_name (str, optional): Logger Name. Defaults to "Template Repository Logger".
        debug (bool, optional): Debugger boolean. Defaults to False.

    Returns:
        logger: return the logger for the current namespace, if it exists. If it does not, create it.
    """

    logger = logging.getLogger(logger_name)
    logger.setLevel(logging.DEBUG)

    # If the logger already has the two handlers we've set, no need to add more.
    if len(logger.handlers) < 2:
        logger.addHandler(get_console_handler(debug))
        if create_log_file != False:
            logger.addHandler(get_file_handler(debug, log_file_name=log_file_name))
        # If debugging is not true, we don't want to output DEBUG information to the console.
        if debug != False:
            coloredlogs.install(
                level="DEBUG",
                logger=logger,
                datefmt=DATE_FMT,
                fmt=FMT,
                field_styles=CUSTOM_FIELD_STYLES,
            )
            logger.debug("Added the debug logger to the console output...")
        else:
            coloredlogs.install(
                level="INFO",
                logger=logger,
                datefmt=DATE_FMT,
                fmt=FMT,
                field_styles=CUSTOM_FIELD_STYLES,
            )

    # With this pattern, it's rarely necessary to propagate the error up to parent.
    logger.propagate = False
    logger.debug("Returning logger to process...")

    return logger



import requests
from .PyTriliumClient import PyTriliumClient


class PyTriliumBranchClient(PyTriliumClient):
    def __init__(self, url, token, debug=False) -> None:
        super().__init__(url, token, debug)

    def get_branch_by_id(self, branch_id: str) -> dict:
        """Given the Branch's ID, this will return the Branch's information.

        Parameters
        ----------
        branch_id : str
            Trilium's ID for the Branch, this can be seen by clicking the 'i' on the branch, near the top.

        Returns
        -------
        requests.Response
            The response from the Trilium API.
        """
        return self.make_request(f"/branches/{branch_id}").json()
    
    def post_branch(self, data: str) -> requests.Response:
        """This will create a new Branch.

        Parameters
        ----------
        data : str
            The data to send to the Trilium API. This should be in the format of a JSON string.

        Returns
        -------
        requests.Response
            The response from the Trilium API.
        """
        return self.make_request("/branches", method="POST", data=data)
    
    def patch_branch_by_id(self, branch_id: str, data: str) -> requests.Response:
        """Given the Branch's ID, this will update the Branch's information.

        Parameters
        ----------
        branch_id : str
            Trilium's ID for the Branch, this can be seen by clicking the 'i' on the branch, near the top.
        data : str
            The data to send to the Trilium API. This should be in the format of a JSON string.

        Returns
        -------
        requests.Response
            The response from the Trilium API.
        """
        return self.make_request(f"/branches/{branch_id}", method="PATCH", data=data)
    
    def delete_branch_by_id(self, branch_id: str) -> requests.Response:
        """Given the Branch's ID, this will delete the Branch.

        Parameters
        ----------
        branch_id : str
            Trilium's ID for the Branch, this can be seen by clicking the 'i' on the branch, near the top.

        Returns
        -------
        requests.Response
            The response from the Trilium API.
        """
        return self.make_request(f"/branches/{branch_id}", method="DELETE")
    